@@ -15,7 +15,7 @@ except:
 # In[3]:
 
 
-version = '0.61'
+version = '0.63'
 
 import re
 import tempfile
@@ -90,7 +90,7 @@ from urllib.request import urlopen
 requirements.append('ipython')
 from IPython.core.display import display, HTML
 
-requirements.append('pip install PyYAML')
+requirements.append('PyYAML')
 import yaml
 
 
@@ -126,6 +126,45 @@ if is_testing():
     print(sFloatSci5(1.234567))
 
 
+# In[110]:
+
+
+def strUuid4():
+    import uuid
+    uuid_with_prefix = uuid.uuid4().urn
+    uuid = uuid_with_prefix[9:]
+    assert len(uuid) == 36
+    return uuid
+
+
+# In[112]:
+
+
+if is_testing():
+    print(strUuid4())
+
+
+# In[113]:
+
+
+def strRand(n, num=True, upperCase=True, lowerCase=True):
+    import random
+    pool = ""
+    if num: pool += '0123456789'
+    if upperCase: pool += 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+    if lowerCase: pool += 'abcdefghijklmnopqrstuvwxyz'
+    return ''.join(random.choice(pool) for _ in range(n))
+
+
+# In[115]:
+
+
+if is_testing():
+    print(strRand(20))
+    print(strRand(20, upperCase=False))
+    print(strRand(20, upperCase=False, lowerCase=False))
+
+
 # # Process / Thread
 
 # In[9]:
@@ -616,7 +655,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[ ]:
+# In[54]:
 
 
 if is_testing():
@@ -624,14 +663,14 @@ if is_testing():
     imshow(img)
 
 
-# In[ ]:
+# In[55]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[ ]:
+# In[56]:
 
 
 def imscaleNN(img, s):
@@ -641,7 +680,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[ ]:
+# In[57]:
 
 
 if is_testing():
@@ -660,7 +699,7 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[ ]:
+# In[58]:
 
 
 def imRepeat(img, height, width):
@@ -674,7 +713,7 @@ def imRepeat(img, height, width):
     return img
 
 
-# In[ ]:
+# In[59]:
 
 
 if is_testing():
@@ -682,33 +721,33 @@ if is_testing():
     imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
 
 
-# In[ ]:
+# In[60]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[ ]:
+# In[61]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[ ]:
+# In[62]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[ ]:
+# In[63]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[ ]:
+# In[64]:
 
 
 def imAddNoiseGauss(img, std):
@@ -720,7 +759,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[ ]:
+# In[65]:
 
 
 def imJpgDegradation(img, quality):
@@ -733,7 +772,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[ ]:
+# In[66]:
 
 
 def imCropCenter(img, size):
@@ -748,35 +787,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[ ]:
+# In[67]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[ ]:
+# In[68]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[ ]:
+# In[69]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[ ]:
+# In[70]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[ ]:
+# In[71]:
 
 
 def imGallery(imgs, pad=0):
@@ -807,7 +846,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[ ]:
+# In[72]:
 
 
 if is_testing():
@@ -815,49 +854,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[ ]:
+# In[73]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[ ]:
+# In[74]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[ ]:
+# In[75]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[ ]:
+# In[76]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[ ]:
+# In[77]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[ ]:
+# In[78]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[ ]:
+# In[79]:
 
 
 if is_testing():
@@ -867,14 +906,14 @@ if is_testing():
 
 # # Files
 
-# In[ ]:
+# In[80]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[ ]:
+# In[81]:
 
 
 if is_testing():
@@ -903,7 +942,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[ ]:
+# In[82]:
 
 
 def fiRemove(path):
@@ -931,7 +970,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[ ]:
+# In[83]:
 
 
 if is_testing():
@@ -963,7 +1002,7 @@ if is_testing():
         print("files: " + "\n".join(fiFindByWildcard(os.path.join(d, "**", "*"))))
 
 
-# In[ ]:
+# In[84]:
 
 
 def fiUnzip(zip_path, out_dir=None, remove=False):
@@ -978,7 +1017,7 @@ def fiUnzip(zip_path, out_dir=None, remove=False):
         zip.extractall(out_dir)
 
 
-# In[ ]:
+# In[85]:
 
 
 def fiJoin(*args):
@@ -990,7 +1029,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[ ]:
+# In[86]:
 
 
 def jnNbFullWidth():
@@ -1004,7 +1043,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', None)
 
 
-# In[ ]:
+# In[87]:
 
 
 def jnImageRenderingPixelated():
@@ -1016,7 +1055,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[ ]:
+# In[88]:
 
 
 def jnDefault():
@@ -1034,7 +1073,7 @@ if is_testing():
 
 
 
-# In[ ]:
+# In[89]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -1047,7 +1086,7 @@ if is_testing():
     
 
 
-# In[ ]:
+# In[90]:
 
 
 def jnImgToHtmlB64(img):
@@ -1082,7 +1121,7 @@ def jnImageRow(imgs, names=None, width=100, external=False, title_img=None, titl
     display(HTML(html))
 
 
-# In[ ]:
+# In[91]:
 
 
 if is_testing():
@@ -1090,7 +1129,7 @@ if is_testing():
    
 
 
-# In[ ]:
+# In[92]:
 
 
 if is_testing():
@@ -1100,7 +1139,7 @@ if is_testing():
     jnImageRow(imgs, external=True, title='test')
 
 
-# In[ ]:
+# In[93]:
 
 
 
@@ -1110,7 +1149,7 @@ if is_testing():
 
 # # Shell
 
-# In[ ]:
+# In[94]:
 
 
 def shell(cmd):
@@ -1125,7 +1164,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[ ]:
+# In[95]:
 
 
 def gitGetHash():
@@ -1135,7 +1174,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[ ]:
+# In[96]:
 
 
 def gitGetBranch():
@@ -1145,7 +1184,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[ ]:
+# In[97]:
 
 
 def gitGetMessage():
@@ -1155,7 +1194,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[ ]:
+# In[98]:
 
 
 def gitGetAllFiles():
@@ -1165,7 +1204,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[ ]:
+# In[99]:
 
 
 def gitListByTime(path=None):
@@ -1181,7 +1220,7 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[ ]:
+# In[100]:
 
 
 def shellSortDirsBySize():
@@ -1193,14 +1232,14 @@ if is_testing():
 
 # # Package Files
 
-# In[ ]:
+# In[101]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[ ]:
+# In[102]:
 
 
 def shell(cmd):
@@ -1208,7 +1247,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[ ]:
+# In[103]:
 
 
 if is_testing():
@@ -1240,7 +1279,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[ ]:
+# In[104]:
 
 
 if is_testing():
@@ -1250,7 +1289,7 @@ description-file = README.md
 """
 
 
-# In[ ]:
+# In[105]:
 
 
 if is_testing():
@@ -1273,14 +1312,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[ ]:
+# In[106]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[ ]:
+# In[107]:
 
 
 if is_testing():
@@ -1289,7 +1328,7 @@ if is_testing():
     
 
 
-# In[ ]:
+# In[108]:
 
 
 if is_testing():
@@ -1300,7 +1339,7 @@ lx/__init__.py
 """
 
 
-# In[ ]:
+# In[109]:
 
 
 if is_testing():
@@ -1317,3 +1356,9 @@ if is_testing():
             #print(shell("cd {d} && git status && git config user.name 'lx' && git config user.email 'lx' && git commit -a -m 'Add' && git log && git push && python setup.py sdist && twine upload dist/* --verbose".format(d=d)))
             print(shell("cd {d} && python3 setup.py sdist && twine upload dist/* --verbose".format(d=d)))
 
+
+# In[ ]:
+
+
+
+
